package ar.com.macro.infraestructure.config;

import ar.com.macro.adapter.aws.secrets.AwsDbSecret;
import ar.com.macro.adapter.aws.secrets.AwsEsbSecret;
import ar.com.macro.infraestructure.exception.NotFoundException;
import camundajar.impl.com.google.gson.Gson;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.AWSStaticCredentialsProvider;
import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.client.builder.AwsClientBuilder;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.secretsmanager.AWSSecretsManager;
import com.amazonaws.services.secretsmanager.AWSSecretsManagerClientBuilder;
import com.amazonaws.services.secretsmanager.model.GetSecretValueRequest;
import com.amazonaws.services.secretsmanager.model.GetSecretValueResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import java.util.Optional;

@Slf4j
@Configuration
@SuppressWarnings("unused")
public class AWSConfig {

    private AWSSecretsManager client;
    @Value("${aws.accessKey}")
    private String accessKey;
    @Value("${aws.secretKey}")
    private String secretKey;
    @Value("${aws.dbSecret}")
    private String dbSecret;
    @Value("${aws.esbSecret}")
    private String esbSecret;
    @Value("${aws.vpc.endpoint}")
    private String awsVpcEndpoint;

    @PostConstruct
    void setupClient() {
        client = AWSSecretsManagerClientBuilder.standard()
                .withCredentials(new AWSStaticCredentialsProvider(credentials()))
                .withRegion(Regions.US_EAST_1).build();
    }

    private AWSCredentials credentials() {
        return new BasicAWSCredentials(this.accessKey, this.secretKey);
    }

    @Bean
    public AwsDbSecret awsDbSecret() {
        return (AwsDbSecret) getSecret(this.dbSecret, AwsDbSecret.class);
    }

    @Bean
    public AwsEsbSecret awsEsbSecret() {
        return (AwsEsbSecret) getSecret(this.esbSecret, AwsEsbSecret.class);
    }

    @Bean
    public AmazonS3 amazonS3() {
        return AmazonS3ClientBuilder.standard()
                .withCredentials(new AWSStaticCredentialsProvider(credentials()))
                .withEndpointConfiguration(
                        new AwsClientBuilder.EndpointConfiguration(this.awsVpcEndpoint, Regions.US_EAST_1.getName()
                        )
                ).build();
    }

    private Object getSecret(String secretName, Class<?> clazz) {
        Gson gson = new Gson();

        String secret;
        GetSecretValueRequest getSecretValueRequest = new GetSecretValueRequest().withSecretId(secretName);
        GetSecretValueResult getSecretValueResult;

        try {
            getSecretValueResult = client.getSecretValue(getSecretValueRequest);
            secret = Optional.ofNullable(getSecretValueResult.getSecretString()).orElseThrow(()
                    -> new NotFoundException("La secret " + secretName + " no existe en la AWS"));
            return gson.fromJson(secret, clazz);
        } catch (Exception e) {
            log.error("Un error ocurrio al intentar buscar la secret {} para la clase {} detalles: {}",
                    secretName, clazz.getName(), e);
            throw e;
        }

    }

}