package ar.com.macro.infraestructure.errorhandling;

import javax.servlet.http.HttpServletRequest;

import org.camunda.bpm.engine.MismatchingMessageCorrelationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import ar.com.macro.infraestructure.exception.BusinessException;
import ar.com.macro.infraestructure.exception.IntegrationException;
import ar.com.macro.infraestructure.exception.NotFoundException;
import ar.com.macro.infraestructure.log.HubLogger;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.experimental.FieldDefaults;

@RestControllerAdvice
@FieldDefaults(level = AccessLevel.PRIVATE)
@AllArgsConstructor(onConstructor_ = @Autowired)
public class DefaultErrorHandler {

    static final HubLogger LOGGER = HubLogger.load(DefaultErrorHandler.class);
    
    HttpResponseResolver errorResponseResolver;

    @ExceptionHandler(Exception.class)
    public ResponseEntity<ErrorResponse> handleException(Exception ex, HttpServletRequest request) {
        LOGGER.logException("unexpected error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.INTERNAL_SERVER_ERROR, ErrorCode.SYSTEM_ERROR, request, ex);
    }

    @ExceptionHandler(HttpMediaTypeNotSupportedException.class)
    public ResponseEntity<ErrorResponse> handleException(HttpMediaTypeNotSupportedException ex, HttpServletRequest request) {
    	LOGGER.logException("http media type not supported error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.UNSUPPORTED_MEDIA_TYPE, ErrorCode.MEDIA_TYPE_NOT_SUPPORTED, request, ex);
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseEntity<ErrorResponse> handleException(HttpMessageNotReadableException ex, HttpServletRequest request) {
        LOGGER.logException("invalid body message error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.BAD_REQUEST, ErrorCode.INVALID_BODY_MESSAGE, request, ex);
    }

    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<ErrorResponse> handleException(MethodArgumentNotValidException ex, HttpServletRequest request) {
    	LOGGER.logException("method argument not valid error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.BAD_REQUEST, request, ex);
    }
    
    @ExceptionHandler(MismatchingMessageCorrelationException.class)
    public ResponseEntity<ErrorResponse> handleException(MismatchingMessageCorrelationException ex, HttpServletRequest request) {
        LOGGER.logException("mismatching message correlation error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.UNPROCESSABLE_ENTITY, ErrorCode.SYSTEM_ERROR, request, ex);
    }

    @ExceptionHandler(IntegrationException.class)
    public ResponseEntity<ErrorResponse> handleException(IntegrationException ex, HttpServletRequest request) {
    	LOGGER.logException("integration error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.FAILED_DEPENDENCY, ErrorCode.DEPENDENCY_FAILED, ex.getMessage(), ex);
    }

    @ExceptionHandler(BusinessException.class)
    public ResponseEntity<ErrorResponse> handleException(BusinessException ex, HttpServletRequest request) {
    	LOGGER.logException("business error", ex);
    	return errorResponseResolver.buildResponse(HttpStatus.UNPROCESSABLE_ENTITY, ErrorCode.BUSINESS_ERROR, ex.getMessage(), ex);
    }
    
    @ExceptionHandler(NotFoundException.class)
    public ResponseEntity<ErrorResponse> handleException(NotFoundException ex, HttpServletRequest request) {
        LOGGER.logException("not found error", ex);
        return errorResponseResolver.buildResponse(HttpStatus.NOT_FOUND, ErrorCode.SYSTEM_ERROR, ex.getMessage(), ex);
    }

}
