package ar.com.macro.infraestructure.errorhandling;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.experimental.FieldDefaults;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Locale;
import java.util.Optional;

@Component
@FieldDefaults(level = AccessLevel.PRIVATE)
@AllArgsConstructor(onConstructor_ = @Autowired)
public class HttpResponseResolver {

    public static final String DEFAULT_LANGUAGE = "pt";

    MessageSource messageSource;

    public ResponseEntity<ErrorResponse> buildResponse(HttpStatus status, ErrorCode errorCode, HttpServletRequest request, Exception ex) {
        String message = getMessage(errorCode.getErrorKey(), request);
        return ResponseEntity.status(status).body(new ErrorResponse(status, ex, getErrorItemResponse(errorCode, message)));
    }

    public ResponseEntity<ErrorResponse> buildResponse(HttpStatus status, ErrorCode errorCode, String message, Exception ex) {
        return ResponseEntity.status(status).body(new ErrorResponse(status, ex, getErrorItemResponse(errorCode, message)));
    }

    public ResponseEntity<ErrorResponse> buildResponse(HttpStatus status, HttpServletRequest request, MethodArgumentNotValidException ex) {
        BindingResult bindingResults = ex.getBindingResult();

        List<ErrorItemResponse> items = bindingResults.getAllErrors().stream().map(e ->
        {
            String defaultMessage = ((DefaultMessageSourceResolvable) e.getArguments()[0]).getDefaultMessage();
            return getErrorItemResponse(
                    ErrorCode.INVALID_FIELD,
                    getMessage(ErrorCode.INVALID_FIELD.getErrorKey(),
                            request, !defaultMessage.isBlank() ? defaultMessage : e.getDefaultMessage())
            );
        }).toList();

        return ResponseEntity.status(status)
                .body(new ErrorResponse(status, ex, items));
    }


    private String getMessage(String descKey, HttpServletRequest request, Object... args) {
        Locale locale = getRequestLocale(request);
        return messageSource.getMessage(descKey, args, locale);
    }

    private Locale getRequestLocale(HttpServletRequest request) {

        String loc = Optional.ofNullable(request.getHeader(HttpHeaders.ACCEPT_LANGUAGE)).orElse(DEFAULT_LANGUAGE);
        if (StringUtils.contains(loc, ",")) {
            loc = StringUtils.substring(loc, 0, loc.indexOf(','));
        }
        return Locale.forLanguageTag(loc);
    }

    private ErrorItemResponse getErrorItemResponse(ErrorCode errorCode, String message) {
        return ErrorItemResponse.builder().code(errorCode.getErrorKey()).message(message).build();
    }

}