package ar.com.macro.infraestructure.log;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import lombok.ToString;
import lombok.experimental.FieldDefaults;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

import static java.util.Arrays.stream;
import static java.util.stream.Collectors.toMap;
import static net.logstash.logback.argument.StructuredArguments.kv;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
@FieldDefaults(level = AccessLevel.PACKAGE)
@ToString
public class HubLogger {

	static final String PARAMETERS = "parameters";
	static final String METHOD_NAME = "methodName";

	Logger logger;

	private static final Map<Class<?>, HubLogger> LOG_CACHE = new HashMap<>();

	public static HubLogger load(Class<?> clazz) {
		return LOG_CACHE.computeIfAbsent(clazz, c -> {

			HubLogger hubLogger = new HubLogger();
			hubLogger.logger = LoggerFactory.getLogger(c);
			return hubLogger;
		});
	}

	public final void log(LogLevel logLevel, String message, Object parameter) {
		logLevel.log(logger, message, kv(PARAMETERS, parameter));
	}

	public final void logInfo(String message, Object object) {
		log(LogLevel.INFO, message, object);
	}

	@SafeVarargs
	public final void logInfo(String message, Pair<String, Object>... params) {
		log(LogLevel.INFO, message, params);
	}

	@SafeVarargs
	public final void log(LogLevel logLevel, String message, Pair<String, Object>... params) {
		logLevel.log(logger, message + " " + formatParametersToString(params));
	}

	private Map<String, Object> listOfPairToMap(Pair<String, Object>... params) {
		return stream(params).collect(toMap(Pair::getKey, Pair::getValue));
	}

	private String formatParametersToString(Pair<String, Object>... params) {
		Map<String, Object> map = listOfPairToMap(params);
		if (MapUtils.isEmpty(map)) {
			return StringUtils.EMPTY;
		}

		return map.toString();
	}

	public void logException(String methodName, Exception e) {
		logException(LogLevel.ERROR, methodName, e);
	}

	public void logException(LogLevel level, String methodName, Exception e) {
		level.log(logger, kv(METHOD_NAME, methodName) + " " + e.getMessage(), e);
	}

}
