package ar.com.macro.adapter.controllers;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.when;

import java.util.List;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import ar.com.macro.adapter.services.AltaComercioService;
import ar.com.macro.adapter.services.ProcesoService;
import ar.com.macro.adapter.services.TycsService;
import ar.com.macro.core.application.incoming.request.AltaComercioRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.core.application.incoming.response.AltaAcuerdoCtaCteResponse;
import ar.com.macro.core.application.incoming.response.AltaComercioResponse;
import ar.com.macro.core.application.incoming.response.CancelacionResponse;
import ar.com.macro.core.application.incoming.response.ConsultaEstadoSolicitudResponse;
import ar.com.macro.core.application.incoming.response.ConsultaTycsResponse;

@ExtendWith(MockitoExtension.class)
class AltaComercioControllerTest {

	@Mock
	private AltaComercioService altaComercioService;

	@Mock
	private ProcesoService procesoService;

	@Mock
	private TycsService tycsService;

	@InjectMocks
	private AltaComercioController altaComercioController;

	@Test
	@DisplayName(
		"Should return a list of consultaestadosolicitudresponse when the request is valid and idsolsmo is null")
	void
	getEstadoSolicitudWhenRequestIsValidAndIdSolSMOIsNull() {
		String canal = "canal";
		String tipoDoc = "tipoDoc";
		Long nroTributario = 1L;
		String idSolicitud = "idSolicitud";
		String idSolSMO = null;

		ResponseEntity<List<ConsultaEstadoSolicitudResponse>> responseEntity =
			altaComercioController.consultaEstadoSolicitud(
				canal, tipoDoc, nroTributario, idSolicitud, idSolSMO);

		Assertions.assertEquals(HttpStatus.OK, responseEntity.getStatusCode());
	}

	@Test
	@DisplayName(
		"Should return a list of consultaestadosolicitudresponse when the request is valid and idsolsmo is not null")
	void
	getEstadoSolicitudWhenRequestIsValidAndIdSolSMOIsNotNull() {
		String canal = "canal";
		String tipoDoc = "tipoDoc";
		Long nroTributario = 1L;
		String idSolicitud = "idSolicitud";
		String idSolSMO = "idSolSMO";

		Mockito.when(altaComercioService.consultarEstado(
			canal, tipoDoc, nroTributario, idSolicitud, idSolSMO))
			.thenReturn(null);

		ResponseEntity<List<ConsultaEstadoSolicitudResponse>> responseEntity =
			altaComercioController.consultaEstadoSolicitud(
				canal, tipoDoc, nroTributario, idSolicitud, idSolSMO);

		Assertions.assertEquals(HttpStatus.OK, responseEntity.getStatusCode());
	}

	@Test
	@DisplayName(
		"Should return a list of consultaestadosolicitudresponse when the request is valid")
	void
	getEstadoSolicitudWhenRequestIsValid() {
		String canal = "canal";
		String tipoDoc = "tipoDoc";
		Long nroTributario = 1L;
		String idSolicitud = "idSolicitud";
		String idSolSMO = "idSolSMO";

		Mockito.when(altaComercioService.consultarEstado(
			canal, tipoDoc, nroTributario, idSolicitud, idSolSMO))
			.thenReturn(null);

		ResponseEntity<List<ConsultaEstadoSolicitudResponse>> responseEntity =
			altaComercioController.consultaEstadoSolicitud(
				canal, tipoDoc, nroTributario, idSolicitud, idSolSMO);

		Assertions.assertEquals(HttpStatus.OK, responseEntity.getStatusCode());
	}

	@Test
	@DisplayName("Should return a response with status ok when the signature is valid")
	void validSignatureWhenSignatureIsValid() {
		FirmaRequest firmaRequest = FirmaRequest.builder().build();
		Mockito.when(altaComercioService.recibirFirma(Mockito.any(), Mockito.any())).thenReturn("123");

		ResponseEntity<AltaComercioResponse> response =
			altaComercioController.validarFirma(firmaRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals("123", response.getBody().getIdSolicitud());
	}

	@Test
	@DisplayName("Should return a response with status ok when the request is valid")
	void startRequestAltaComercioWhenRequestIsValid() {
		AltaComercioRequest altaComercioRequest = AltaComercioRequest.builder().build();
		String processInstanceId = "123";
		
		Mockito.when(altaComercioService.iniciar(altaComercioRequest)).thenReturn(processInstanceId);
		Mockito.when(procesoService.getEstadoDelProceso("123")).thenReturn("estadodeteste");

		ResponseEntity<AltaComercioResponse> response =
			altaComercioController.iniciar(altaComercioRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals(HttpStatus.OK.name(), response.getBody().getResultado());
		Assertions.assertEquals(HttpStatus.OK.value(), response.getBody().getCodResultado());
		Assertions.assertEquals(HttpStatus.OK.series().name(), response.getBody().getDescripcion());
		Assertions.assertEquals(processInstanceId, response.getBody().getIdSolicitud());
		Assertions.assertEquals("estadodeteste", response.getBody().getEstadoDelProceso());
	}

	@Test
	@DisplayName("Should return a response when call cancellation with status ok when the request is valid")
	void cancelAltaComercioWhenRequestIsValid() {
		CancelacionRequest cancelationRequest = CancelacionRequest.builder().build();

		Mockito.doAnswer(mocked-> {
			return null;
		}).when(altaComercioService).cancelacion(Mockito.any(), Mockito.any());

		ResponseEntity<CancelacionResponse> response =
				altaComercioController.cancelacion(cancelationRequest);

		Assertions.assertEquals(HttpStatus.OK, response.getStatusCode());
		Assertions.assertEquals(HttpStatus.OK.series().name(), response.getBody().getDescripcion());
	}

	@Test
	@DisplayName("Should return tyc with status ok")
	void getTycsAltaComercio() {
		final ResponseEntity<ConsultaTycsResponse> expectedResult = new ResponseEntity<>(
			new ConsultaTycsResponse("OK", 200, "SUCCESSFUL", "idSolicitud", List.of(0)), HttpStatus.OK);
		when(tycsService.getTycsPorIdSolicitud("idSolicitud")).thenReturn(List.of(0));

		final ResponseEntity<ConsultaTycsResponse> result = altaComercioController.consultaTycs("idSolicitud");

		assertEquals(expectedResult, result);
	}

}