package ar.com.macro.adapter.services;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.camunda.bpm.engine.RuntimeService;
import org.camunda.bpm.engine.history.HistoricProcessInstance;
import org.camunda.bpm.engine.impl.persistence.entity.HistoricProcessInstanceEntity;
import org.camunda.bpm.engine.runtime.MessageCorrelationBuilder;
import org.junit.Assert;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.bpm.scope.AltaAcuerdoCtaCteScopeHelper;
import ar.com.macro.bpm.scope.AltaComercioScopeHelper;
import ar.com.macro.core.application.incoming.request.AltaComercioRequest;
import ar.com.macro.core.application.incoming.request.CancelacionRequest;
import ar.com.macro.core.application.incoming.request.FirmaRequest;
import ar.com.macro.core.application.incoming.response.ConsultaEstadoSolicitudResponse;
import ar.com.macro.infraestructure.exception.BusinessException;
import ar.com.macro.infraestructure.exception.NotFoundException;

@ExtendWith(MockitoExtension.class)
class AltaComercioServiceTest {

    private static final String PROCESS_INSTANCE_ID = "607846f6-7ca0-11ed-b4c1-c6598dc64a4a";
    private static final String ESTADO = "created";
    
    @Mock
    private ProcesoService procesoService;
    
    @Mock
    private RuntimeService runtimeService;
    
    @Mock
    private FirmaRequest firmaRequest;
    
    @Mock
    private MessageCorrelationBuilder correlation;
    
    @InjectMocks
    private AltaComercioService altaComercioService;

    @Test
    @DisplayName("Should throw an exception when the process is not found")
	void recibirFirmaAltaComercioWhenProcessIsNotFoundThenThrowException() {
    	
		Mockito.when(procesoService.buscaProcessos(any(), any(), any(), any(), any())).thenReturn(new ArrayList<>());
		assertThrows(NotFoundException.class, () -> altaComercioService.consultarEstado("canal", "tipoDoc", 1L, "idSolicitud", "idSolSMO"));
	}

    @Test
    @DisplayName("Should throw an exception when the request is invalid")
	void startAltaComercioWhenRequestIsInvalidThenThrowException() {
    	
		Assertions.assertThrows(Exception.class, () -> {
			altaComercioService.iniciar(null);
		});
	}

    @Test
    @DisplayName("Should return the process id when the request is valid")
    void startAltaComercioWhenRequestIsValidThenReturnProcessId() {
    	
        AltaComercioRequest altaComercioRequest = AltaComercioRequest.builder().build();
        Mockito.when(procesoService.ejecutar(any(), any())).thenReturn(PROCESS_INSTANCE_ID);
        String processId = altaComercioService.iniciar(altaComercioRequest);
        Assert.assertEquals(PROCESS_INSTANCE_ID, processId);
    }

	@Test
	void getEstadoSolicitudAltaComercio() {
		
		List<HistoricProcessInstance> historicVariableInstances = new ArrayList<>();
		HistoricProcessInstanceEntity historicVariableInstanceEntity = new HistoricProcessInstanceEntity();
		historicVariableInstanceEntity.setId(PROCESS_INSTANCE_ID);
		historicVariableInstances.add(historicVariableInstanceEntity);
		Mockito.when(procesoService.buscaProcessos(any(), any(), any(), any(), any()))
			.thenReturn(historicVariableInstances);
		List<ConsultaEstadoSolicitudResponse> historic = altaComercioService.consultarEstado(any(), any(), any(), any(), any());
		Mockito.verify(procesoService).buscaProcessos(any(), any(), any(), any(), any());
		Assert.assertEquals(1, historic.size());
		Assert.assertEquals(PROCESS_INSTANCE_ID, historic.get(0).getIdSolicitud());
		Assert.assertEquals(ESTADO, historic.get(0).getEstado());
	}

    @Test
    void getRecebirFirmaAltaComercio() {

        MessageCorrelationBuilder messageCorrelationBuilder = Mockito.mock(MessageCorrelationBuilder.class);
        Mockito.doReturn(messageCorrelationBuilder).when(this.procesoService).createMessageCorrelation(anyString());

        Mockito.doReturn(UUID.randomUUID().toString()).when(firmaRequest).getIdSolicitud();
        
        String s = altaComercioService.recibirFirma(firmaRequest, new AltaComercioScopeHelper(null));
        assertNotNull(s);
    }

    @Test
    void cancelAltaComercioShouldThrowException() {
    	
        CancelacionRequest cancelacionRequest = CancelacionRequest.builder()
                .idCanal("canal123")
                .idSolicitud(UUID.randomUUID().toString())
                .motivoCancelacion("Cancelamento de la Solicitud")
                .cuitFirmante(Long.valueOf(14582))
                .build();

        MessageCorrelationBuilder messageCorrelationBuilder = Mockito.mock(MessageCorrelationBuilder.class);

        Mockito.doReturn(messageCorrelationBuilder).when(this.procesoService).createMessageCorrelation(anyString());
        Mockito.doReturn(true).when(this.procesoService).isActivityInstance(anyString());

        try {
        	altaComercioService.cancelacion(cancelacionRequest, new AltaComercioScopeHelper(null));
        } catch (Exception e) {
        	assertEquals(BusinessException.class, e.getClass());
        }
    }
    
    @Test
    void cancelAltaComercioShouldSuccess() {
    	
        CancelacionRequest cancelacionRequest = CancelacionRequest.builder()
                .idCanal("canal123")
                .idSolicitud(UUID.randomUUID().toString())
                .motivoCancelacion("Cancelamento de la Solicitud")
                .cuitFirmante(Long.valueOf(14582))
                .build();
    	
        MessageCorrelationBuilder messageCorrelationBuilder = Mockito.mock(MessageCorrelationBuilder.class);
        
        Mockito.doReturn(messageCorrelationBuilder).when(this.procesoService).createMessageCorrelation(anyString());
        Mockito.doReturn(false).when(this.procesoService).isActivityInstance(anyString());

		Assertions.assertDoesNotThrow(() -> {
			altaComercioService.cancelacion(cancelacionRequest, new AltaComercioScopeHelper(null));
		});
    }
	
}
