package ar.com.macro.adapter.services;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.UUID;

import org.junit.Assert;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

import ar.com.macro.bpm.scope.DesbordePrestamoScopeHelper;
import ar.com.macro.bpm.scope.ScopeHelper;
import ar.com.macro.core.application.incoming.request.CuotaRequest;
import ar.com.macro.core.application.incoming.request.DesbordePrestamoIniciarRequest;
import ar.com.macro.core.application.incoming.response.DesbordePrestamoDetalleResponse;
import ar.com.macro.infraestructure.exception.NotFoundException;

@ExtendWith(MockitoExtension.class)
class DesbordePrestamoServiceTest {
	@Mock
	private ProcesoService procesoService;
	@InjectMocks
	private DesbordePrestamoService desbordePrestamoService;

	@Test
	@DisplayName("Should throw an exception when the request is invalid")
	void startWhenRequestIsInvalidThenThrowException() {
		assertThrows(
			Exception.class,
			() -> {
				desbordePrestamoService.iniciar(null);
			});
	}

	@Test
	@DisplayName("Should return the process id when the request is valid")
	void startWhenRequestIsValidThenReturnProcessId() {
		final String PROCESS_INSTANCE_ID = UUID.randomUUID().toString();
		DesbordePrestamoIniciarRequest desbordePrestamoIniciarRequest = DesbordePrestamoIniciarRequest.builder().build();
		when(procesoService.ejecutar(any(), any())).thenReturn(PROCESS_INSTANCE_ID);
		String processId = desbordePrestamoService.iniciar(desbordePrestamoIniciarRequest);
		Assert.assertEquals(PROCESS_INSTANCE_ID, processId);
	}

	@Test
	@DisplayName("Should return the process id")
	void startValidRequest() {
        var request = DesbordePrestamoIniciarRequest
                .builder()
                .canal("123")
                .cuit(1)
                .paso("123")
                .razonSocial("123")
                .productoId(1)
                .productoDesc("123")
                .fecha(LocalDateTime.now())
                .valorDisponible(1.0)
                .valorCalificado(1.0)
                .montoSolicitado(1.0)
                .perPago("123")
                .plazo(1)
                .cuotas(1)
                .valorCuotas(1.0)
                .tasaFija("123")
                .cft("123")
                .gastos(1.0)
                .sistemaAmortizacion("123")
                .cuenta(1l)
                .tipoCuenta("123")
                .cuentaDesenbolso(1l)
                .moneda("123")
                .tipoDocumento("123")
                .numeroDoc(1l)
                .tipoPrestamo("123")
                .numeroCliente(1)
                .lineaComercial(1)
                .sublineaComercial(1)
                .mercado(1)
                .submercado(1)
                .cuota(Arrays.asList(CuotaRequest
                        .builder()
                        .numero(1)
                        .vencimiento(LocalDateTime.now())
                        .capitalAdeudado(1.0)
                        .capital(1.0)
                        .intereses(1.0)
                        .seguro(1.0)
                        .rubros(1.0)
                        .iva(1.0)
                        .totalPagar(1.0)
                        .build()))
								.apellidoSolicitante("teste")
								.mailSolicitante("teste@teste.com")
								.nombreSolicitante("testing")
								.telefonoSolicitante(3223232l)
                .build();
		desbordePrestamoService.iniciar(request);
		verify(procesoService).ejecutar(any(), any());
	}
	
    @Test
    @DisplayName("Should return detalle query")
    void consultaDetalle() {
        String idSolicitud = "cd9c6f81-9cad-11ed-94ca-00ff05749dfc";
        when(procesoService.isActivityInstance(idSolicitud)).thenReturn(true);
        	
		DesbordePrestamoDetalleResponse processId = this.desbordePrestamoService.consultaDetalle(idSolicitud);

		assertNotNull(processId);
    }
    
    @Test
    @DisplayName("Should throw an exception when the process is not found")
    void shouldReturnException() {
    	String idSolicitud = null;
    	
    	Mockito.when(procesoService.isActivityInstance(idSolicitud)).thenReturn(false);
    	
		assertThrows(NotFoundException.class, () -> desbordePrestamoService.consultaDetalle(idSolicitud));
    }
    
	@Test
	@DisplayName("checks if cuotasItem is different from null")
    void verificaCuotasItemsDiferenteDeNull() {

		String idSolicitud = UUID.randomUUID().toString();

		Mockito.when(procesoService.isActivityInstance(idSolicitud)).thenReturn(true);
    	
        when(procesoService.isActivityInstance(idSolicitud)).thenReturn(true);
        when(procesoService.getVariable(idSolicitud, ScopeHelper.CANAL)).thenReturn("canal");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.NUMERO_CLIENTE)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUIT)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_DOCUMENTO)).thenReturn("tipoDocumento");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.NUMERO_DOC)).thenReturn(1L);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PRODUCTO_ID)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PRODUCTO_DESC)).thenReturn("productoDesc");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PASO)).thenReturn("paso");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.FECHA)).thenReturn(LocalDateTime.now());
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_DISPONIBLE)).thenReturn(1D);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MONTO_SOLICITADO)).thenReturn(1D);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PER_PAGO)).thenReturn("perPago");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_PRESTAMO)).thenReturn("tipoPrestamo");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.PLAZO)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUOTAS)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_CUOTAS)).thenReturn(1D);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TASA_FIJA)).thenReturn("tasaFija");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CFT)).thenReturn("cft");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.GASTOS)).thenReturn(1D);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SISTEMAS_AMORTIZACION)).thenReturn("sistemasAmortizacion");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUENTA)).thenReturn(1L);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.TIPO_CUENTA)).thenReturn("tipoCuenta");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUENTA_DESENBOLSO)).thenReturn(1L);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MONEDA)).thenReturn("moneda");
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.LINEA_COMERCIAL)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SUB_LINEA_COMERCIAL)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.MERCADO)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.SUB_MERCADO)).thenReturn(1);
        when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.VALOR_CALIFICADO)).thenReturn(1.0);
        
        Mockito.when(procesoService.getVariable(idSolicitud, DesbordePrestamoScopeHelper.CUOTA)).thenReturn(new ArrayList<>(Arrays.asList(CuotaRequest
                .builder()
                .numero(1)
                .vencimiento(LocalDateTime.now())
                .capitalAdeudado(1.0)
                .capital(1.0)
                .intereses(1.0)
                .seguro(1.0)
                .rubros(1.0)
                .iva(1.0)
                .totalPagar(1.0)
                .build())));

		desbordePrestamoService.consultaDetalle(idSolicitud);
		
        Mockito.verify(this.procesoService, Mockito.times(30))
        .getVariable(Mockito.any(), Mockito.any());
        
        Mockito.verify(this.procesoService, Mockito.times(1))
        .isActivityInstance(idSolicitud);

	}
}
